@@ -79,6 +79,9 @@
 #define EDITOR  		"vi"  /* <- evangelism! */
 #define u32 u_int32_t
 
+#define OPT_DACL	0x98
+#define OPT_SACL	0x99
+
 static int apply_action(const char *, const struct stat *, int, struct FTW *);
 static int do_apply_action(const char *, const struct stat *);
 static int open_editor(const char *);
@@ -110,6 +113,8 @@ static struct option long_options[] = {
 	{ "recursive",		0, 0, 'R' },
 	{ "physical",		0, 0, 'P' },
 	{ "logical",		0, 0, 'L' },
+	{ "dacl",		0, 0, OPT_DACL },
+	{ "sacl",		0, 0, OPT_SACL },
 	{ NULL,			0, 0, 0,  },
 };
 
@@ -124,6 +129,8 @@ static char *mod_string;
 static char *from_ace;
 static char *to_ace;
 
+static enum acl_type acl_type = ACL_TYPE_ACL;
+
 /* XXX: things we need to handle:
  *
  *  - we need some sort of 'purge' operation that completely clears an ACL.
@@ -136,7 +143,7 @@ int main(int argc, char **argv)
 	int opt, err = 1;
 	int numpaths = 0, curpath = 0;
 	char *tmp, **paths = NULL, *path = NULL, *spec_file = NULL;
-	FILE *s_fp = NULL;
+	FILE *s_fp, *fd = NULL;
 
 	if (!strcmp(basename(argv[0]), "nfs4_editfacl")) {
 		action = EDIT_ACTION;
@@ -148,7 +155,7 @@ int main(int argc, char **argv)
 		return err;
 	}
 
-	while ((opt = getopt_long(argc, argv, "-:a:A:s:S:x:X:m:ethvHRPL", long_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "-:a:A:i:s:S:x::X:m:ethvHRPL", long_options, NULL)) != -1) {
 		switch (opt) {
 			case 'a':
 				mod_string = optarg;
@@ -158,21 +165,14 @@ int main(int argc, char **argv)
 			add:
 				assert_wu_wei(action);
 				action = INSERT_ACTION;
-
-				/* run along if no more args (defaults to ace_index 1 == prepend) */
-				if (optind == argc)
-					break;
-				ace_index = strtoul_reals(argv[optind++], 10);
-				if (ace_index == ULONG_MAX) {
-					/* oops it wasn't an ace_index; reset */
-					optind--;
-					ace_index = -1;
-				} else if (ace_index == 0) {
-					fprintf(stderr, "Sorry, valid indices start at '1'.\n");
-					goto out;
+				break;
+			case 'i':
+				ace_index = strtoul_reals(optarg, 10);
+				if (ace_index == 0) {
+                                    fprintf(stderr, "Sorry, valid indices start at '1'.\n");
+                                    goto out;
 				}
 				break;
-
 			case 's':
 				mod_string = optarg;
 				goto set;
@@ -184,9 +184,14 @@ int main(int argc, char **argv)
 				break;
 
 			case 'x':
-				ace_index = strtoul_reals(optarg, 10);
-				if(ace_index == ULONG_MAX)
-					mod_string = optarg;
+				/* make sure we handle the argument even if
+				 * it doesn't immediately follow the option
+				 */
+				if (optarg == NULL && optind < argc && argv[optind][0] != '-')
+				{
+					optarg = argv[optind++];
+				}
+				mod_string = optarg;
 				goto remove;
 			case 'X':
 				spec_file = optarg;
@@ -248,6 +253,9 @@ int main(int argc, char **argv)
 					case 'A':
 						fprintf(stderr, "Sorry, -a requires an 'acl_spec', whilst -A requires a 'spec_file'.\n");
 						goto out;
+					case 'i':
+						fprintf(stderr, "Sorry, -i requires an index (numerical)\n");
+						goto out;
 					case 's':
 						fprintf(stderr, "Sorry, -s requires an 'acl_spec'.\n");
 						goto out;
@@ -272,6 +280,13 @@ int main(int argc, char **argv)
 				paths[numpaths++] = optarg;
 				break;
 
+			case OPT_DACL:
+				acl_type = ACL_TYPE_DACL;
+				break;
+			case OPT_SACL:
+				acl_type = ACL_TYPE_SACL;
+				break;
+
 			case 'h':
 			case '?':
 			default:
@@ -283,7 +298,21 @@ int main(int argc, char **argv)
 	if (action == NO_ACTION) {
 		fprintf(stderr, "No action specified.\n");
 		goto out;
-	} else if (numpaths < 1) {
+	} else if (action != INSERT_ACTION && action != REMOVE_ACTION && ace_index >= 0) {
+		fprintf(stderr, "Index can be used only with add or remove.\n");
+		goto out;
+	} else if (numpaths <= 0 && ace_index >= 0 && mod_string)
+	{
+		/* Make sure the argument is a file */
+		if (!(fd = fopen(mod_string, "r"))) {
+			fprintf(stderr, "No path(s) specified.\n");
+			goto out;
+		} else
+			fclose(fd);
+		paths = malloc(sizeof(char *) * (argc - optind + 1));
+		paths[numpaths++] = mod_string;
+	} else if (numpaths < 1)
+	{
 		fprintf(stderr, "No path(s) specified.\n");
 		goto out;
 	}
@@ -334,6 +363,50 @@ out:
 	return err;
 }
 
+static void nfs4_print_acl_error(const char *path)
+{
+	switch (errno) {
+	case ENODATA:
+		fprintf(stderr,"Attribute not found on file: %s\n", path);
+		break;
+	case EREMOTEIO:
+		fprintf(stderr,"An NFS server error occurred.\n");
+		break;
+	case EOPNOTSUPP:
+		fprintf(stderr,"Operation to request attribute not supported: "
+			       "%s\n", path);
+		break;
+	default:
+		perror("Failed operation");
+	}
+}
+
+static struct nfs4_acl *nfs4_retrieve_acl(const char *path,
+					  enum acl_type type)
+{
+	switch (type) {
+	case ACL_TYPE_DACL:
+		return nfs4_getdacl(path);
+	case ACL_TYPE_SACL:
+		return nfs4_getsacl(path);
+	default:
+		return nfs4_getacl(path);
+	}
+}
+
+static int nfs4_apply_acl(const char *path, struct nfs4_acl *acl,
+			  enum acl_type type)
+{
+	switch (type) {
+	case ACL_TYPE_DACL:
+		return nfs4_setdacl(path, acl);
+	case ACL_TYPE_SACL:
+		return nfs4_setsacl(path, acl);
+	default:
+		return nfs4_setacl(path, acl);
+	}
+}
+
 /* returns 0 on success, nonzero on failure */
 static int apply_action(const char *_path, const struct stat *stat, int flag, struct FTW *ftw)
 {
@@ -378,7 +451,7 @@ static int do_apply_action(const char *path, const struct stat *_st)
 	if (action == SUBSTITUTE_ACTION)
 		acl = nfs4_new_acl(S_ISDIR(st->st_mode));
 	else
-		acl = nfs4_acl_for_path(path);
+		acl = nfs4_retrieve_acl(path, acl_type);
 
 	if (acl == NULL) {
 		fprintf(stderr, "Failed to instantiate ACL.\n");
@@ -438,8 +511,11 @@ static int do_apply_action(const char *path, const struct stat *_st)
 	if (is_test) {
 		fprintf(stderr, "## Test mode only - the resulting ACL for \"%s\": \n", path);
 		nfs4_print_acl(stdout, acl);
-	} else
-		err = nfs4_set_acl(acl, path);
+	} else {
+		err = nfs4_apply_acl(path, acl, acl_type);
+		if (err == -1)
+			nfs4_print_acl_error(path);
+	}
 
 out:
 	nfs4_free_acl(acl);
@@ -548,9 +624,10 @@ static void __usage(const char *name, int is_ef)
 	"%s %s -- manipulate NFSv4 file/directory access control lists\n"
 	"Usage: %s [OPTIONS] COMMAND file ...\n"
 	" .. where COMMAND is one of:\n"
-	"   -a acl_spec [index]	 add ACL entries in acl_spec at index (DEFAULT: 1)\n"
-	"   -A file [index]	 read ACL entries to add from file\n"
-	"   -x acl_spec | index	 remove ACL entries or entry-at-index from ACL\n"
+	"   -a acl_spec		 add ACL entries in acl_spec at defaul index (DEFAULT: 1)\n"
+	"   -A file 		 read ACL entries to add from file\n"
+	"   -i index 		 use the entry-at-index from ACL (only for add and remove)\n"
+	"   -x acl_speci 	 remove ACL entries\n"
 	"   -X file  		 read ACL entries to remove from file\n"
 	"   -s acl_spec		 set ACL to acl_spec (replaces existing ACL)\n"
 	"   -S file		 read ACL entries to set from file\n"
